from arduino import Arduino
import time
import csv
from apm import *

import matplotlib.pyplot as plt
plt.ion()


b = Arduino('COM3')
write_pin = 3
read_pin = 'A0'


#Declare output pins as a list/tuple
b.output([])

#Setup output file
output_file = open('output.csv', 'wb')
file_writer = csv.writer(output_file)
'''
#Setup graph
x = []
yI = []
yS = []
plt.plot(x, yI)
plt.plot(x, yS)
plt.xlabel('Time')
plt.ylabel('Temperature')
plt.ylim(60,150)
'''

#Initialize
startTime = time.time()
lastTime = int(round(time.time() * 1000))
lastErr = 0
errSum = 0
out_min = 0
out_max = 255
out = 0

cycle_time = 2

################################
#######  APM  ##################
################################

#Initialize server
s = 'http://byu.apmonitor.com'
app = 'mpctest'

apm(s,app,'clear all')
apm_load(s,app,'mpc.apm')
apm_info(s,app,'CV','DegF')
apm_info(s,app,'MV','voltage')
csv_load(s,app,'mpc.csv')

apm_option(s,app,'nlc.imode',6)

apm(s,app,'solve')
apm_option(s,app,'nlc.reqctrlmode',3)
apm_option(s,app,'nlc.web_plot_freq',2)
apm_option(s,app,'DegF.FSTATUS',1)
apm_option(s,app,'voltage.FSTATUS',0)
apm_option(s,app,'DegF.status',1)
apm_option(s,app,'voltage.status',1)
apm_option(s,app,'voltage.lower',0)
apm_option(s,app,'voltage.upper',255)
apm_option(s,app,'DegF.splo',110)
apm_option(s,app,'DegF.sphi',113)
apm_web(s,app)

################################
################################
################################

print "Initializing..."

#Run Program
try:
    while (True):
        
        #Read
        time.sleep(1)
        print "---------------"
        print "INPUT"
        raw_input = b.analogRead(read_pin)
        print raw_input
        if(raw_input != "w"):
            val = int(raw_input)
            print val
            voltage = val*(3300/1024)
            print "voltage: %s" % voltage
            DegC = (voltage - 500)/10
            DegF = DegC*1.8+32
            read_out = 'DegC: ' + str(DegC) + ' DegF: ' + str(DegF)
            print read_out
        else:
            print "ERROR"

        ###########################################
        #### CONSTRUCT CONTROLLER HERE ############
        ###########################################
        '''
        Available Variables
        Control:
            setpoint
            kp
            ki
            kd
        Input:
            voltage
        Output:
            out
        '''

        #PID Control
        print '\n MPC:'
        
        #Get delta t
        now = int(round(time.time() * 1000))
        timeChange = now - lastTime
        print ' Delta t: %s' % timeChange
        time.sleep(min(0,2-timeChange/1000))

        #Insert into MPC
        apm_meas(s,app,'DegF',DegF)
        apm(s,app,'solve')
        out = int(apm_tag(s,app,'voltage.newVal'))
        print "Out: %s" % out

        print "Did MPC"
       
        lastTime = now


        ###########################################
        #######   END CONTROLLER  #################
        ###########################################

        #Limit output to 255 mV
        if out > out_max:
            out = out_max
        if out < out_min:
            out = out_min
        print 'Out Limited: %s' % out
 

        #Write
        print "Writing to file:"
        print "Write pin: %s" % write_pin
        print "Out: %s" % out
        b.analogWrite(write_pin, out)
        print "Wrote to file:"
        print ''

        
        '''
        #Update Graph
        x.append(elapsed_time)
        yI.append(DegF)
        yS.append(setpoint)
        plt.plot(x, yI, color='r')
        plt.plot(x, yS, color='b')
        plt.ylim(60,150)
        plt.draw()
        '''
        '''
        #Digital Write
        b.setHigh(write_pin)
        time.sleep(.1)
        print b.getState(write_pin)
        b.setLow(write_pin)
        print "Blink"
        time.sleep(.1)
        '''

#End Program with Ctrl C
except KeyboardInterrupt:
    print "\nExiting..."
    b.analogWrite(write_pin, 0)
    #plt.show()
    b.close()
    output_file.close()

